"""Some code to deal with the FUNQL geo880 data

Data comes from the Kwiatkowski et al. EMNLP 2010 UBL paper.
"""
import collections
import glob
import os
import random
import re
import sys

IN_DIR = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))),
    'data/geo880/funql-raw')
ORIG_DIR = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))),
    'data/geo880/processed-lesscopy')
OUT_DIR = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))),
    'data/geo880/processed-funql')

def read_examples(filename):
  with open(filename) as f:
    lines = [x.strip() for x in f if not x.startswith('//') and x.strip()]
  return zip(lines[0::2], lines[1::2])

def split_logical_form(lf):
  # denver:e -> " denver "
  # san_francisco:e -> " san francisco "
  # _:e -> " _ "
  lf = lf.replace('_:e', '" _ "')
  lf = re.sub('([_a-z]*)\:e', lambda m: '" %s "' % m.group(1).replace('_', ' '), lf)
  replacements = [
      ('(', ' ( _'),
      (')', ' ) '),
  ]
  for a, b in replacements:
    lf = lf.replace(a, b)
  return ' '.join(lf.split())

def write(data, filename):
  with open(filename, 'w') as f:
    for x, y in data:
      print >> f, '%s\t%s' % (x, y)

def process(filename, stemmer=None, less_copy=False):
  random.seed(1)
  print >> sys.stderr, 'Processing %s' % filename
  basename = os.path.basename(filename)
  domain = 'geo880'
  stage = basename.split('.')[1]
  
  in_data = read_examples(filename)
  out_data = []
  for (utterance, logical_form) in in_data:
    y = split_logical_form(logical_form)
    out_data.append((utterance, y))

  out_basename = '%s_%s%d.tsv' % (domain, stage, len(out_data))
  out_path = os.path.join(OUT_DIR, out_basename)
  write(out_data, out_path)

  # Do an additional small train/dev split
  if stage == 'train':
    random.shuffle(out_data)
    train_data = out_data[:500]
    dev_data = out_data[500:]

    train_basename = '%s_%s%d.tsv' % (domain, 'train', len(train_data))
    train_path = os.path.join(OUT_DIR, train_basename)
    write(train_data, train_path)

    dev_basename = '%s_%s%d.tsv' % (domain, 'dev', len(dev_data))
    dev_path = os.path.join(OUT_DIR, dev_basename)
    write(dev_data, dev_path)

def main():
  if not os.path.exists(OUT_DIR):
    os.makedirs(OUT_DIR)
  for filename in sorted(glob.glob(os.path.join(IN_DIR, 'geo880-funql.*'))):
    process(filename)

if __name__ == '__main__':
  main()
